# Order - API


Este projeto tem como objetivo integrar os pedidos delivery de parceiros ao PDV Teknisa.
A integração funciona de forma autônoma, desde o recebimento do pedido no parceiro até o envio do mesmo a cozinha do restaurante.

## Funcionamento Credito Pessoal

### Regras de funcionamento:

- Não existe gerenciamento de saldo através do Order-Api, sendo de sua responsabilidade apenas processar o pedido e integra-lo caso o consumidor tenha saldo igual ou superior ao pedido.
- O order-api não realiza debito de saldo, sendo esta operação executada no PDV no momento da emissão da venda
- Devido ao restrição de não debitar saldo, o order-api não permite a integração de um novo pedido com pagamento com crédito pessoal caso exista pedidos delivery em aberto realizado nas   últimas 6 horas.
- O tipo de recebimento enviado no campo "paymentId" obrigatóriamente precisar ser relacionado à um tipo de recebimento do tipo Credito Pessoal na tela Cadastro de Integração->Order Api-> Tipo de Recebimento (Retail)


### Consulta dados do consumidor:

  #### A consulta dos dados do consumidor é feita através do endpoint getConsumerById, sendo obrigatório o envio do cracha do consumidor como payload através do campo "consumerRef".


> É necessário consultar os dados do consumidor para obter os dados de cadastro do consumidor no sistema Teknisa, sendo eles:
> * "clientRef" -> Código de cliente vinculado ao consumidor,
> * "clientId"  -> Cracha vinculado ao consumidor


&nbsp;


<details>
  <summary>
    <p><b>WebHook para recepção de alterações no status do pedido</b></p>
  </summary> 

### Estrura do evento:

- **Body:**
    ```Json
    {
        "orderId": "0001-0000123717-0000113491-35",
        "orderType": "DELIVERY",
        "origin": "EATTK",
        "status": "STATUS_PREPARATION",
        "eventData": "2023-09-21T00:00:00Z"
    }
    ```

- ## Legenda:
  
| Param     | Description                                   |
| --------- | --------------------------------------------- |
| orderId   | Identificador único do pedido                 |
| orderType | Tipo de pedido Delivery (Entrega ou retirada) |
| origin    | Origem de pedido (Plataforma de integração).  |
| status    | Status do pedido                              |
| eventData | Data e hora do evento                         |


&nbsp;


| Status             | Description                                        |
| ------------------ | -------------------------------------------------- |
| STATUS_ACCEPTED    | Pedido Aceito                                      |
| STATUS_PREPARATION | Pedido em preparação                               |
| STATUS_ON_ROUTE    | Pedido saiu para entrega (Saída do entregador PDV) |
| STATUS_DELIVERED   | Pedido Entregue (Chegada do entregador PDV)        |
| STATUS_FINISHED    | Pedido Finalizado                                  |
| STATUS_CANCELLED   | Data e hora do evento                              |


> Necessário encaminhar solicitação de ativação do serviço através do email delivery@teknisa.com, enviar junto ao email o DNS que os eventos devem ser direcionados.


&nbsp;

</details>


<details>
  <summary>
    <p><b>Configurações de ambiente</b></p>
  </summary>

## Framework
- [Slim v3](https://www.slimframework.com/docs/v3/) 

## Dependências
### Backend
- Composer - [composer.json](http://gitlab.teknisa.com/odhen/integrations/order-api/-/blob/master/backend/composer.json)

## Arquivos Estruturais
### Backend
#### Arquivos de Configuração
- [env.ini](http://gitlab.teknisa.com/odhen/integrations/order-api/-/blob/master/backend/env.ini) - Responsável pelo apontamento de qual cliente a integração deverá considerar em sua execução
- [configs.json](http://gitlab.teknisa.com/odhen/integrations/order-api/-/blob/master/backend/presets/configs/configs_cloud9.json)
- [config.json](http://gitlab.teknisa.com/odhen/integrations/order-api/-/blob/master/backend/presets/clients/external/cloud9/config.json) - Responsável por definir as credenciais de acesso a API, Apontamento de SMTP e diretório de logs 
- [environment.json](http://gitlab.teknisa.com/odhen/integrations/order-api/-/blob/master/backend/presets/clients/external/cloud9/environment.json) - Reponsável pelas configurações de conexão com o banco de dados (OracleDB)


## Instalação
### Desenvolvimento
1. Com o projeto clonado no seu ambiente Cloud, acesse a pasta ***Backend*** e altere o arquivo [env.ini] com as seguintes informações:
```ini
EXTERNAL_CLIENT_PRESET=cloud9
```

2. Acesse a pasta ***Backend/Configs*** e altere o arquivo [configs.json] com a seguintes informações:
```json
{
    "client"     : "",
    "server"     : "cloud9",
    "logPath"    : "/home/developer/workfolder/logs/orderapi",

    "host"       : "smtp.gmail.com",
    "username"   : "odhensm@gmail.com",
    "from"       : "odhensm@gmail.com",
    "password"   : "OOT0OnaCvj1abDya2aHPqGXs2xA4KK1qdM6Sk0/8T/8D+ihymflpaxgvsfmTpn/dyDYBXyZpb8x/wJthwCVdBw==",
    "SMTPSecure" : "tls",
    "port"       : 587,
    
    "timezone"   : "SP"
}
```
3. Acesse a pasta ***backend/presets/clients/*** copie a pasta [(cloud9)] junto de seus sub-diretórios e cole dentro de ***backend/clients/external***. 

**OBS: Importante manter somente a pasta cloud9 no diretorio "backend/clients/external" afim de evitar que erro no apontamento de cliente gere maiores impactos.**

4. Pelo terminal, acesse a pasta ***backend*** e execute o seguinte comando:
```Bash
composer install
```
</details>

&nbsp;


## Endpoints

<details>
  <summary>
    <code>GET</code>
    <code><b>/</b></code>
    <p><b>Rota de validação de status e versão da API</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}

    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "client": String, 
        "version": String,
        "revision": String,
        "date": String,
        "filialCount": Integer
    }
    ```
</details>


________________________________________________________________________________________________________________________________



<details>
  <summary>
    <code>POST</code>
    <code><b>/order</b></code>
    <p><b>Rota para envio de pedidos delivery</b></p>
  </summary>

#### Request
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```
- **Body**
    ```Json
    {
        "saleId": "{{key_pedido_parceiro}}",
        "orderNumber": "{{identificador__pedido_parceiro}}",
        "orderType": 1,
        "origin": "PLAT1",
        "date": "2024-04-24 13:00:15",
        "scheduleDate": "",
        "tableNumber": "",
        "total": 12.5,
        "troco": 0,
        "subTotal": 7.50,
        "subsidiaryCode": "0001",
        "tax": 5,
        "discount": 0,
        "latitude": 0,
        "longitude": 0,
        "cpfInNote": null,
        "notes": "Pedido Teste - Não Produzir",
        "consumerDetail": {
            "clientId": "000000000000000000002",
            "name": "Consumidor Teste",
            "email": "consumertest@system.com",
            "cpf": "11111111111",
            "phone": "31999999999",
            "cellPhone": "31999998888",
            "consumerRef": "INTEGRACAO",
            "clientRef": "O00001",
            "address": {
                "zipCode": "11111111",
                "street": "Rua teste",
                "number": "123",
                "neighborhood": "Savassi",
                "city": "Belo Horizonte",
                "uf": "MG",
                "complement": "Condominio XPTO",
                "referencePoint": "Proximo a praca XPTO"
            }
        },
        "payments": [
            {
                "paymentId": "DIN",
                "value": 12.5
            }
        ],
        "ItemOrder": [
            {
                "title": "Sanduiche da Tek",
                "quantity": 1,
                "price": 7.5,
                "total": 7.5,
                "notes": "Enviar guardanapos",
                "ComplementCategories": [
                    {
                        "title": "Molho da Tek",
                        "priceUn": 0,
                        "total": 0,
                        "quantity": 1,
                        "notes": "",
                        "ref": "01|01"
                    }
                ],
                "ref": "9150100501"
            }
        ],
        "CombinedProducts": [
            {
                "title": "Grande 2 sabores (6 Fatias)",
                "quantity": 1,
                "price": 0,
                "total": 1,
                "notes": "  1x Massa Tradicional  1x Tradicional",
                "ComplementCategories": [],
                "combination": [
                    {
                        "title": "Pizza Calabresa",
                        "price": 0.5,
                        "total": 1,
                        "quantity": 1,
                        "notes": "",
                        "ref": "9100502001"
                    },
                    {
                        "title": "Pizza 4 queijos",
                        "price": 0.5,
                        "total": 1,
                        "quantity": 1,
                        "notes": "",
                        "ref": "9100502001"
                    },
                    {
                        "title": "Massa Tradicional",
                        "price": 0,
                        "total": 0,
                        "quantity": 1,
                        "notes": "",
                        "ref": ""
                    }
                ],
                "ref": "01"
            }
        ]
    }
    ```
______________________________________________________
#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "success": true,
        "message": "Pedido integrado com sucesso.",
        "result": {
            "Order": "0001-0000012058-0000012515-89"
        }
    }
    ```


- ## Legenda:
| Param            | Required | Description                                                                                                    |
| ---------------- | -------- | -------------------------------------------------------------------------------------------------------------- |
| saleId           | true     | Identificador único do pedido na plataforma de terceiros                                                       |
| orderNumber      | true     | Identificador resumido do pedido na plataforma de terceiros. (4 digitos)                                       |
| orderType        | true     | Tipo do pedido delivery (1) ou retirada na loja (2).                                                           |
| origin           | true     | Origem do pedido. Identificação da plataforma de terceiros - 5 caracteres - Ex: PLA01 (Plataforma Ficticia 1). |
| date             | true     | Data que o pedido foi realizado na plataforma.                                                                 |
| scheduleDate     | true     | Data para agendar o pedido dentro do sistema (enviar somente em pedidos agendados).                            |
| subsidiaryCode   | true     | Código da filial no sistema Teknisa                                                                            |
| storeCode        | true     | Código de loja no sistema Teknisa                                                                              |
| posCode          | true     | Código do caixa no sistema Teknisa                                                                             |
| tableNumber      | true     | Código da mesa no sistema Teknisa                                                                              |
| tax              | true     | Taxa de entrega do pedido (enviar 0 quando não houver).                                                        |
| subTotal         | true     | Somatoria dos produtos + acréscimos.                                                                           |
| total            | true     | Total do pedido (Subtotal + taxa de entrega - descontos).                                                      |
| troco            | true     | Troco para' do pedido.                                                                                         |
| discount         | true     | Valor de desconto total que será rateado entre os itens do pedido.                                             |
| latitude         | true     | Latitude do local de entrega do pedido.                                                                        |
| longitude        | true     | Longitude do local de entrega do pedido.                                                                       |
| cpfInNote        | true     | CPF para impressão na nota.                                                                                    |
| notes            | true     | Observação geral do pedido (máximo: 100 caracteres).                                                           |
| consumerDetail   | true     | <a class="consumer-details">Objeto: Detalhes do Consumidor - Detalhes do consumidor</a>                        |
| payments         | true     | Objeto: Detalhes do Pagamento realizado.                                                                       |
| ItemOrder        | true     | Objeto: Detalhes do Pedido para produto Simples.                                                               |
| CombinedProducts | true     | Objeto: Detalhes do Pedido para produto Combinados (Pizzas).                                                   |


<details class="consumer-details">
  <summary>
    <b>Consumer Details - Detalhes do consumidor (Objeto)</b>
  </summary>

| Param       | Required | Description                                                                                                  |
| ----------- | -------- | ------------------------------------------------------------------------------------------------------------ |
| clientId    | true     | Identificador único do consumidor na plataforma de terceiros.                                                |
| name        | true     | Nome do consumidor (máximo: 50).                                                                             |
| email       | true     | E-mail do consumidor (máximo: 50).                                                                           |
| cpf         | true     | CPF do consumidor (enviar vazio quando não existir)                                                          |
| phone       | true     | telefone fixo do consumidor (enviar vazio quando não existir)                                                |
| cellPhone   | true     | telefone celular do consumidor (enviar vazio quando não existir)                                             |
| consumerRef | false    | Identificador do consumidor na Teknisa (Cracha)                                                              |
| clientRef   | false    | Identificador de cliente vinculado ao consumidor                                                             |
| address     | true     | Objeto: Endereço do consumidor completo. (enviar vazio para pedidos com modalidade to go / retirada na loja) |

<details>
<summary>
    <b>Address - Endereço do Consumidor (Objeto)</b>
</summary>

| Param          | Required | Description                                                             |
| -------------- | -------- | ----------------------------------------------------------------------- |
| zipCode        | true     | CEP do consumidor.                                                      |
| street         | true     | Rua do consumidor (máximo: 50).                                         |
| number         | true     | Número do consumidor.                                                   |
| neighborhood   | true     | Bairro do consumidor (máximo: 100).                                     |
| city           | true     | Cidade do consumidor.                                                   |
| uf             | true     | Sigla do estado.                                                        |
| complement     | true     | Complemento do consumidor (enviar vazio quando não existir).            |
| referencePoint | true     | Referência do endereço do consumidor (enviar vazio quando não existir). |
</details>

</details>


<details>
  <summary>
    <b>Payments - Métodos de pagamento utilizados no pedido (objeto)</b>
  </summary>

| Param     | Required | Description                                            |
| --------- | -------- | ------------------------------------------------------ |
| paymentId | true     | Identificador de pagamento na plataforma de terceiros. |
| value     | true     | Valor do pagamento.                                    |

</details>

<details>
  <summary>
    <b>ItemOrder - Produtos (objeto)</b>
  </summary>

| Param                | Required | Description                                                               |
| -------------------- | -------- | ------------------------------------------------------------------------- |
| title                | true     | Nome do Item.                                                             |
| quantity             | true     | Quantidade do Item.                                                       |
| price                | true     | Preço unitário do Item principal.                                         |
| total                | true     | Total do Item (somado ao preço dos acréscimos, quando houver).            |
| notes                | true     | Observação escrita para o Item.                                           |
| ref                  | true     | Código de produto do Item.                                                |
| ComplementCategories | true     | Complementos do item (Acréscimos, Retiradas e/ou Observações Adicionais). |

<details>
  <summary>
    <b>ComplementCategories - Complementos do Item (objeto)</b>
  </summary>

| Param    | Required | Description                                                    |
| -------- | -------- | -------------------------------------------------------------- |
| title    | true     | Nome do Item.                                                  |
| quantity | true     | Quantidade do Item.                                            |
| priceUn  | true     | Preço unitário do Item principal.                              |
| total    | true     | Total do Item (somado ao preço dos acréscimos, quando houver). |
| notes    | true     | Observação escrita para o Item.                                |
| ref      | true     | Código de produto do Item.                                     |

</details>
</details>


<details>
  <summary>
    <b>CombinedProducts - Produtos combinados (Pizzas com combinação de sabores) (objeto)</b>
  </summary>

| Param                | Required | Description                                                    |
| -------------------- | -------- | -------------------------------------------------------------- |
| title                | true     | Nome do Item.                                                  |
| quantity             | true     | Quantidade do Item.                                            |
| price                | true     | Preço unitário do Item principal.                              |
| total                | true     | Total do Item (somado ao preço dos acréscimos, quando houver). |
| notes                | true     | Observação escrita para o Item.                                |
| ref                  | true     | Código de produto do Item.                                     |
| ComplementCategories | true     | Enviar array vazio                                             |
| combination          | true     | Combinações do item (Sabores e tipo de massa).                 |

<details>
  <summary>
    <b>combination - Combinações do item (objeto)</b>
  </summary>

| Param    | Required | Description                                                    |
| -------- | -------- | -------------------------------------------------------------- |
| title    | true     | Nome do Item.                                                  |
| quantity | true     | Quantidade do Item.                                            |
| price    | true     | Preço unitário do Item principal.                              |
| total    | true     | Total do Item (somado ao preço dos acréscimos, quando houver). |
| notes    | true     | Observação escrita para o Item.                                |
| ref      | true     | Código de produto do Item.                                     |

</details>
</details>
</details>

________________________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/orderStatus</b></code>
    <p><b>Consulta o status do pedido</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```
- **Body**
    ```Json
    {
        "orderId": "0001-0000012101-0000012558-317"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "orderStatus": "PRODUCAO"
    }
    ```

- ## Legenda:
| Param   | Required | Description                                                            |
| ------- | -------- | ---------------------------------------------------------------------- |
| orderId | true     | Identificador único do pedido na na Teknisa (Retornado na rota /order) |


- ## Mapeamento de Status:
| Status     | Description                                         |
| ---------- | --------------------------------------------------- |
| PRODUCAO   | Pedido integrado e em fase de produção              |
| CANCELADO  | Pedido cancelado                                    |
| EXPEDIDO   | Saiu para entrega                                   |
| IMPRESSO   | Cupom fiscal emitido, aguardando saída para entrega |
| FINALIZADO | Concluído                                           |

</details>

________________________________________________________________________________________________________________________________

<details>
  <summary>
    <code>POST</code>
    <code><b>/orderCancel</b></code>
    <p><b>Cancela pedido - Apenas pedidos com status "PRODUCAO"</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```
- **Body**
    ```Json
    {
        "orderId": "0001-0000012101-0000012558-317"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "success": 1
    }
    ```

- ## Legenda:
| Param   | Required | Description                                                            |
| ------- | -------- | ---------------------------------------------------------------------- |
| orderId | true     | Identificador único do pedido na na Teknisa (Retornado na rota /order) |

</details>

________________________________________________________________________________________________________________________________


<details>
  <summary>
    <code>GET</code>
    <code><b>/getBranchs</b></code>
    <p><b>Retorna lojas da organização por filial</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    [
        {
            "branchId": "0001",
            "branchName": "ODHEN INTEGRACOES",
            "branchDefaultStore": "01",
            "stores": [
                {
                    "storeId": "01",
                    "storeName": "ODHEN INTEGRACOES"
                }
            ]
        }
    ]
    ```

- ## Legenda:
| Param              | Description                              |
| ------------------ | ---------------------------------------- |
| branchId           | Código da filial na Teknisa              |
| branchName         | Nome da Filial                           |
| branchDefaultStore | Código de loja definida como padrão      |
| stores             | Lojas cadastradas para a filial (Objeto) |


<details>
  <summary>
    <b>Lojas cadastradas para a filial (Objeto)</b>
  </summary>

| Param     | Description  |
| --------- | ------------ |
| storeId   | Código       |
| storeName | Nome da Loja |

</details>

</details>


________________________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/menu</b></code>
    <p><b>Retorna cardápio  da Filial</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```

- **Body**
    ```Json
    {
        "branchId": "0001"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "success": true,
        "menu": {
            "Products": [
                {
                    "groupName": "COMBOS",
                    "products": [
                        {
                            "itemName": "COMBO DO COMBO IMP NAO",
                            "itemId": "9101001503",
                            "itemModifierId": "3",
                            "itemValue": 0,
                            "itemSubProducts": [
                                {
                                    "subProductGroupName": "BURGERS",
                                    "subProductGroupId": "0000000005",
                                    "subProductGroupLimit": "3",
                                    "subProductGroupMin": "1",
                                    "subProductGroupItems": [
                                        {
                                            "subItemName": "MOCARNE E QUEIJO 120GR",
                                            "subItemId": "9000100100",
                                            "subItemValue": 22.9,
                                            "subItemDiscountValue": 0,
                                            "subItemDiscountType": "P"
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                },
                {
                    "groupName": "PIZZAS",
                    "pizzaProducts": [
                        {
                            "pizzaName": null,
                            "pizzaValue": null,
                            "pizzaLimit": null,
                            "pizzaTaxMethod": null,
                            "products": []
                        },
                        {
                            "pizzaName": "PIZZAS G",
                            "pizzaValue": 0,
                            "pizzaLimit": "2",
                            "pizzaTaxMethod": "M",
                            "products": [
                                {
                                    "subItemName": "PIZZA A MODA G",
                                    "subItemId": "9100503000",
                                    "productId": "9100503000",
                                    "description": "Massa de pizza feita com fermentação natural, molho de tomate rústico, mussarela tradicional, presunto, azeitona verde e orégano.",
                                    "value": 39.99
                                }
                            ]
                        }
                    ]
                }
            ],
            "Observations": [
                {
                    "01": {
                        "16": {
                            "O": {
                                "observationName": "COM GELO",
                                "products": [
                                    "9150100100",
                                    "9150100101",
                                    "9150100102"
                                ]
                            },
                            "A": {
                                "observationName": "ADICIONAR 1",
                                "observationId": "9101501003",
                                "observationValue": 2,
                                "products": [
                                    "9000100100",
                                    "9000100101",
                                    "9000100102"
                                ]
                            },
                            "R": {
                                "observationName": "SEM ACUCAR",
                                "products": [
                                    "9051001000",
                                    "9051002000"
                                ]
                            }
                        }
                    }
                }
            ]
        }
    }
    ```

- ## Legenda:
| Param   | Description                 |
| ------- | --------------------------- |
| success | Houve erro ou não           |
| menu    | cardápio da filial (Objeto) |



<details>
  <summary>
    <b>Menu de produtos (Objeto)</b>
  </summary>

| Param        | Description                                           |
| ------------ | ----------------------------------------------------- |
| Products     | Grupos de produtos do menu (Objeto)                   |
| Observations | Grupos de Ocorrencias possiveis por produtos (Objeto) |

<details>
  <summary>
    <b>Grupos de produtos do menu (Objeto)</b>
  </summary>

| Param         | Description                                 |
| ------------- | ------------------------------------------- |
| groupName     | Nome do grupo de produto                    |
| products      | produtos do grupo (Objeto)                  |
| pizzaProducts | Pizzas disponiveis para combinação (Objeto) |



<details>
  <summary>
    <b>produtos do grupo (Objeto)</b>
  </summary>

| Param           | Description                                        |
| --------------- | -------------------------------------------------- |
| itemName        | Nome do Produto                                    |
| itemId          | Código do produto (Utilizado para envio de pedido) |
| itemModifierId  | Tipo do produto (3 para combo)                     |
| itemValue       | valor do produto                                   |
| itemSubProducts | Grupos promocionais vinculados ao produto (Objeto) |


<details>
  <summary>
    <b>Grupos promocionais vinculados ao produto (Objeto)</b>
  </summary>

| Param                | Description                                   |
| -------------------- | --------------------------------------------- |
| subProductGroupName  | Nome do grupo promocional                     |
| subProductGroupId    | Código do do grupo promocional                |
| subProductGroupLimit | Quantidade máxima de escolhas dentro do grupo |
| subProductGroupMin   | Quantidade mínima de escolhas dentro do grupo |
| subProductGroupItems | Produtos vinculados ao grupo (Objeto)         |


<details>
  <summary>
    <b>Produtos vinculados ao grupo (Objeto)</b>
  </summary>

| Param                | Description                                              |
| -------------------- | -------------------------------------------------------- |
| subItemName          | Nome do produto                                          |
| subItemId            | Código do produto (Utilizado para envio de pedido)       |
| subItemValue         | Valor do produto                                         |
| subItemDiscountValue | Valor de desconto que precisa calcular ao produto        |
| subItemDiscountType  | Tipo de calculo do desconto (P - Percentual / V - Valor) |


</details>
</details>
</details>


<details>
  <summary>
    <b>Pizzas disponiveis para combinação (Objeto)</b>
  </summary>

| Param          | Description                                                                        |
| -------------- | ---------------------------------------------------------------------------------- |
| pizzaName      | Nome do Produto (Ex: Pizzas Grandes)                                               |
| pizzaValue     | Valor da Pizza (enviado com valor 0)                                               |
| pizzaLimit     | Quantidade de sabores à escolher                                                   |
| pizzaTaxMethod | M - Considera preço do sabor mais caro / C - Consedera soma dos valores por fração |
| products       | Grupos promocionais vinculados ao produto (Objeto)                                 |


<details>
  <summary>
    <b>Pizzas disponiveis para combinação (Objeto)</b>
  </summary>

| Param       | Description                                        |
| ----------- | -------------------------------------------------- |
| subItemName | Nome do Sabor                                      |
| subItemId   | Código do produto (Utilizado para envio de pedido) |
| productId   | (Utilizado para envio de pedido)                   |
| description | Descrição do produto                               |
| value       | Valor cheio do sabor                               |

</details>
</details>
</details>


<details>
  <summary>
    <b>Grupos de Ocorrencias possiveis por produtos (Objeto)</b>
  </summary>

| Param         | Description                             |
| ------------- | --------------------------------------- |
| {{cod_grupo}} | Código do grupo de Ocorrencias (Objeto) |

<details>
  <summary>
    <b>Código do grupo de Ocorrencia (Objeto)</b>
  </summary>

| Param              | Description                   |
| ------------------ | ----------------------------- |
| {{cod_observacao}} | Código da Ocorrencia (Objeto) |


<details>
  <summary>
    <b>Código da Ocorrencia (Objeto)</b>
  </summary>

| Param | Description                                     |
| ----- | ----------------------------------------------- |
| O     | Se Ocorrencia for do tipo "Observação" (Objeto) |
| R     | Se Ocorrencia for do tipo "Retirada"   (Objeto) |
| A     | Se Ocorrencia for do tipo "Acrescimo"  (Objeto) |


<details>
  <summary>
    <b>Se Ocorrencia for do tipo "Observação" (Objeto)</b>
  </summary>

| Param           | Description                                    |
| --------------- | ---------------------------------------------- |
| observationName | Nome de ocorrencia                             |
| products        | Array de produtos que utilizam esta ocorrência |

</details>

<details>
  <summary>
    <b>Se Ocorrencia for do tipo "Retirada" (Objeto)</b>
  </summary>

| Param           | Description                                    |
| --------------- | ---------------------------------------------- |
| observationName | Nome de ocorrencia                             |
| products        | Array de produtos que utilizam esta ocorrência |

</details>


<details>
  <summary>
    <b>Se Ocorrencia for do tipo "Acrescimo" (Objeto)</b>
  </summary>

| Param            | Description                                     |
| ---------------- | ----------------------------------------------- |
| observationName  | Nome de ocorrencia                              |
| observationId    | Código do produto que deve adicionado ao pedido |
| observationValue | Valor do produto adicionado                     |
| products         | Array de produtos que utilizam esta ocorrência  |

</details>

</details>
</details>
</details>
</details>
</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>GET</code>
    <code><b>/getMenus</b></code>
    <p><b>Retorna os caixas e validade de cardápio por filial/loja</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    [
        {
            "branchId": "0001",
            "branchName": "ODHEN INTEGRACOES",
            "branchDefaultStore": "01",
            "stores": [
                {
                    "storeId": "01",
                    "storeName": "ODHEN INTEGRACOES",
                    "pos": [
                        {
                            "posId": "001",
                            "posName": "CAIXA 01",
                            "posType": "Restaurante/Comanda/Delivery com Venda Balcão",
                            "menus": [
                                {
                                    "menuId": "00001",
                                    "validFrom": "23/12/21"
                                }
                            ]
                        }
                    ]
                }
            ],
            "defaultPos": "009"
        }
    ]
    ```

- ## Legenda:

| Param              | Description                                    |
| ------------------ | ---------------------------------------------- |
| branchId           | Código da filial na Teknisa                    |
| branchName         | Nome Da Filial                                 |
| defaultPos         | Código de caixa definido como padrão da filial |
| branchDefaultStore | Código de loja definido como padrão da filial  |
| stores             | Lojas da filial (Objeto)                       |


<details>
  <summary>
    <b>Lojas da filial (Objeto)</b>
  </summary>

| Param     | Description                       |
| --------- | --------------------------------- |
| storeId   | Código da loja                    |
| storeName | Nome da loja                      |
| pos       | caixas vinculados a loja (Objeto) |


<details>
  <summary>
    <b>Lojas da filial (Objeto)</b>
  </summary>

| Param   | Description                                      |
| ------- | ------------------------------------------------ |
| posId   | Código do caixa                                  |
| posName | Nome do caixa                                    |
| posType | Modalidades ativa para o caixa                   |
| menus   | Cardápios vinculados ao caixa (Array de Objetos) |


<details>
  <summary>
    <b>Cardápios vinculados ao caixa (Array de Objetos)</b>
  </summary>

| Param     | Description          |
| --------- | -------------------- |
| menuId    | Código de cardápio   |
| validFrom | Validade do cardápio |


</details>
</details>
</details>

</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/getProducts</b></code>
    <p><b>Retorna lista de produtos vinculados ao menu da filial e produtos bloqueados (Somente tabela de preço geral)</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```

- **Body:**
    ```Json
    {
        "branchId": "0001",
        "menuId": "00001"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "branchId": "0001",
        "menuId": "00001",
        "products": [
            {
                "itemId": "9051002500",
                "itemName": "Coca Cola lata 350ml",
                "itemValue": 3.5
            }
        ],
        "blockedProducts": [
            "9051001000",
            "9051002000"
        ]
    }
    ```

- ## Legenda:

| Param           | Description                                      |
| --------------- | ------------------------------------------------ |
| branchId        | Código da filial na Teknisa                      |
| menuId          | Código do Cardápio (Consultado na rota getMenus) |
| blockedProducts | Array com os produto bloqueados                  |
| products        | Produtos vinculados ao cardápio (Objeto)         |



<details>
  <summary>
    <b>Produtos vinculados ao cardápio (Objeto)</b>
  </summary>

| Param     | Description       |
| --------- | ----------------- |
| itemId    | Código do produto |
| itemName  | Nome do produto   |
| itemValue | Valor do produto  |


</details>
</details>
</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>GET</code>
    <code><b>/getPricingTables</b></code>
    <p><b>Retorna tabelas de preços cadastradas (Apenas tabela de preços geral)</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    [
        {
            "tableId": "001",
            "tableName": "tabela geral 1",
            "validFrom": "23/05/2022",
            "validUntil": "31/12/2099",
            "branches": [
                {
                    "branchId": "0001",
                    "branchName": "ODHEN INTEGRACOES"
                }
            ]
        }
    ]
    ```

- ## Legenda:

| Param      | Description                                    |
| ---------- | ---------------------------------------------- |
| tableId    | Código da tabela de preços                     |
| tableName  | Nome da tabela de preços                       |
| validFrom  | Data de início de validade da tabela de preços |
| validUntil | Data final de validade da tabela de preços     |
| branches   | Filiais associadas à tabela de preços (Objeto) |


<details>
  <summary>
    <b>Filiais associadas à tabela de preços (Objeto)</b>
  </summary>

| Param      | Description      |
| ---------- | ---------------- |
| branchId   | Código da Filial |
| branchName | Nome da Filial   |


</details>
</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/getProductsByPricingTable</b></code>
    <p><b>Retorna eventos de alterações no cardápio (Apenas tabela de preços geral)</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```

- **Body:**
    ```Json
    {
        "tableId": "0001"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "tableId": "001",
        "products": [
            {
                "itemId": "9000100101",
                "itemName": "X-BACON 120GR",
                "itemValue": 25.9
            }
        ]
    }
    ```

- ## Legenda:

| Param    | Description                                       |
| -------- | ------------------------------------------------- |
| tableId  | Código da tabela de preços                        |
| products | Produtos cadastrados na tabela de preços (Objeto) |

<details>
  <summary>
    <b>Produtos cadastrados na tabela de preços (Objeto)</b>
  </summary>

| Param     | Description       |
| --------- | ----------------- |
| itemId    | Código do produto |
| itemName  | Nome do produto   |
| itemValue | Valor do produto  |


</details>
</details>

____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>GET</code>
    <code><b>/getMenusUpdates</b></code>
    <p><b>Retorna eventos de alterações no cardápio </b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    [
        {
            "date": "07/06/2024 16:15:23",
            "branch": "00001",
            "description": "NRCONFTELA: 00001 IDTPBUTTON: 5 NRBUTTON: 01 NRPGCONFTELA: 1 IDTPBUTONAUX: 6 NRBUTTONAUX: 06 CDIDENTBUTON: 017 DSBUTTON: PIX NRCOLORTEXT: 13750737 NRCOLORBACK: 5254654 NRPGCONFTAUX: 1 DSMESSAGEBUT:  DTULTATUITCT: 07/06/2024 16:15:23 NRFONTETEXT: 8 CDATAPRODITM:  CDNATPRODUTO:  NRLIMPRODCOM:  DSENDEIMG:  IDTPBUTONAUX2: 0 NRBUTTONAUX2: 0 NRPGCONFTAUX2: 0 IDMODOMENU:  DSENDEIMGAUX:  DSITMENUVENDA:  IDEXIBEAPPCONS: S DSBUTTONINGLES:  DSBUTTONESPANH:  IDSHOWTAA: S DTINIVIGENCIA: 23/12/2021 00:00:00"
        }
    ]
    ```

- ## Legenda:

| Param       | Description                  |
| ----------- | ---------------------------- |
| date        | Data e hora de alteração     |
| branch      | Código da filial             |
| description | Detalhes do que foi alterado |

</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/getBranchData</b></code>
    <p><b>Retorna informações da filial</b></p>
  </summary>

#### Obs:
- **Caso "branchId" seja enviado com valor vazio ou nulo, será retornado as informações de todas as filiais.** 

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```
- **Body:**
    ```Json
    {
        "branchId": "0001"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    {
        "data": [
            {
                "branchId": "0001",
                "tradingName": "ODHEN INTEGRACOES",
                "companyName": "ODHEN INTEGRACOES",
                "employerNumber": "19874652001821",
                "stateRegistration": "0963924834",
                "address": "Avenida Assis Brasil, 8787",
                "addressComplement": "lj",
                "neighborhood": "SARANDI",
                "city": "PORTO ALEGRE",
                "uf": "RS",
                "contactName": null,
                "tel": "31 2122 2331",
                "email": "delivery@teknisa.com",
                "initialDate": "07/07/2017",
                "activeId": "S"
            }
        ]
    }
    ```

- ## Legenda:

| Param | Description                    |
| ----- | ------------------------------ |
| data  | Informações da filial (Objeto) |

<details>
  <summary>
    <b>Informações da filial (Objeto)</b>
  </summary>

| Param             | Description              |
| ----------------- | ------------------------ |
| branchId          | Código da filial         |
| tradingName       | Nome Fantasia            |
| companyName       | Razão Social             |
| employerNumber    | Incrição juridica CNPJ   |
| stateRegistration | Incrição Estadual        |
| address           | Endereço                 |
| addressComplement | Complemento de endereço  |
| neighborhood      | Bairro                   |
| city              | Cidade                   |
| uf                | Estado                   |
| contactName       | Nome para contato        |
| tel               | Telefone de contato      |
| email             | Email da filial          |
| initialDate       | Data de Cadastro         |
| activeId          | Status de cadastro (S/N) |


</details>
</details>


____________________________________________________________________________________________________________________


<details>
  <summary>
    <code>POST</code>
    <code><b>/getConsumerById</b></code>
    <p><b>Retorna informações do consumidor</b></p>
  </summary>

#### Request:
- **Header:**
    ```
    Authorization: Bearer {{token}}
    Content-Type: application/json
    ```

- **Body:**
    ```Json
    {
        "consumerRef": "INTEGRACAO"
    }
    ```

#### Response:
- **status HTTP:**
    200
- **Body:**
    ```Json
    [
        {
            "clientId": "000000000000000000002",
            "name": "ZE DO CREDITO",
            "email": "",
            "phone": "31986834520",
            "cellPhone": "",
            "cpf": "",
            "consumerRef": "INTEGRACAO",
            "clientRef": "O00001"
        }
    ]
    ```

- ## Legenda:

| Param       | Description                                                  |
| ----------- | ------------------------------------------------------------ |
| clientId    | Código de consumidor no sistema Teknisa                      |
| name        | Nome do consumidor no sistema Teknisa                        |
| phone       | Email do consumidor no sistema Teknisa                       |
| cellPhone   | Celular do consumidor no sistema Teknisa                     |
| cpf         | CPF do consumidor no sistema Teknisa                         |
| consumerRef | Cracha do consumidor no sistema Teknisa                      |
| clientRef   | Código do cliente vinculado ao consumidor no sistema Teknisa |

</details>
