<?php

namespace App\Factory;

use App\Exception;

class BranchFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getStandardClient($branch) {
        $sql = "
            SELECT CDCLIENTE
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ":CDFILIAL" => $branch
        ];
        
        $stdClient = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($stdClient["CDCLIENTE"])) 
            throw new Exception\ParamsException("Cliente padrão não encontrado para a filial - $branch");
        
        return $stdClient;
    }
    
    public function getStandardConsumerType($branch) {
        $sql = "
            SELECT CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ":CDFILIAL" => $branch    
        ];
        
        $consumerT = $this->connection->fetchAll($sql, $params)[0];
        
        if(empty($consumerT["CDTIPOCONS"]))
            throw new Exception\ParamsException("Tipo de consumidor padrão não encontrado para a filial - $branch");
        
        if(empty($consumerT["CDCCUSCLIE"]))
            throw new Exception\ParamsException("Centro de custo do cliente não encontrado para a filial - $branch");
        
        return $consumerT;
    }
    
    public function getAddressInfo ($branch) {
        $sql = "
            SELECT CDMUNICIPIO, SGESTADO, DSENDEFILI, NRENDEFILISEP, NMBAIRFILI, NRCEPFILI
              FROM ENDEFILI
             WHERE CDFILIAL     = :CDFILIAL
               AND IDTPENDEFILI = 'P'";
        
        $params = [
            ":CDFILIAL" => $branch    
        ];
        
        return $this->connection->fetchAll($sql, $params)[0];
    }
   
   public function getCityName($sgEstado,$cdMunicipio){
       $sql = "
            SELECT NMMUNICIPIO
              FROM MUNICIPIO
             WHERE CDPAIS        = :CDPAIS 
               AND SGESTADO      = :SGESTADO
               AND CDMUNICIPIO   = :CDMUNICIPIO";
        
        $params = [
            ":CDPAIS"       =>  "0055",
            ":SGESTADO"     =>  $sgEstado,
            ":CDMUNICIPIO"  =>  $cdMunicipio
        ];
        return $this->connection->fetchAssoc($sql, $params);
   } 
   
   
    public function getBranchsNames() {
        $sql = "
            SELECT CDFILIAL, NMFILIAL
              FROM FILIAL
          ORDER BY CDFILIAL
        ";
        
        return $this->connection->fetchAll($sql);
   } 
   
   public function getMenuBranches($menuId) {
        $sql = "
            SELECT T.CDFILIAL, F.NMFILIAL
              FROM FILITABGER T
              JOIN FILIAL F
                ON F.CDFILIAL = T.CDFILIAL
             WHERE T.CDTABEPREC = :CDTABEPREC
          ORDER BY F.CDFILIAL
        ";
        
        $params = [
            ":CDTABEPREC" => $menuId
        ];
        
        return $this->connection->fetchAll($sql, $params);
    } 
   
    public function getBranchData($cdFilial) {
        $sql = "
            SELECT 
                F.CDFILIAL,     /*Código da loja*/
                F.NMFILIAL,     /*Nome fantasia*/
                F.NMRAZSOCFILI, /*Razão Social*/
                F.NRINSJURFILI, /*CNPJ*/
                F.CDINSCESTA,   /*Insc. Estadual*/
                EF.DSENDEFILI,  /*Endereço*/
                EF.DSCOMPENDFIL,/*Complemento*/
                EF.NMBAIRFILI,  /*Bairro*/
                M.NMMUNICIPIO,  /*Cidade*/
                EF.SGESTADO,    /*UF*/
                EF.NMCONTFILI,  /*Nome do contato*/
                EF.NRTELEFILI,  /*telefone da loja*/
                EF.DSEMAILFILI, /*e-mail da loja*/
                TO_CHAR(F.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO,   /*Data inicial*/
                F.IDATIVO       /*Ativa: Sim, Não*/
            FROM FILIAL F 
                JOIN ENDEFILI EF 
                    ON (F.CDFILIAL = EF.CDFILIAL)
                JOIN MUNICIPIO M
                    ON (EF.CDMUNICIPIO = M.CDMUNICIPIO)
            WHERE (:CDFILIAL IS NULL OR F.CDFILIAL = :CDFILIAL)
            AND EF.IDTPENDEFILI	= 'P'
            ORDER BY CDFILIAL ASC
        ";
        
        $params = [
            ":CDFILIAL" => $cdFilial
        ];
        
        return $this->connection->fetchAll($sql, $params);
   } 

}
